@@ -1,4 +1,3 @@
-import asyncio
 import configparser
 import json
 import logging
@@ -6,7 +5,7 @@ import os
 import pathlib
 from dataclasses import replace
 from decimal import Decimal, InvalidOperation
-from typing import IO, Any, Dict, Mapping, Optional, Tuple
+from typing import IO, Dict, Optional, Tuple
 
 import click
 import yaml
@@ -15,14 +14,20 @@ from prompt_toolkit.patch_stdout import patch_stdout
 from rich.markup import escape as rich_escape
 
 from neuro_sdk import (
-    Preset,
     _Balance,
+    _CloudProviderType,
     _Cluster,
     _ClusterUserRoleType,
     _ConfigCluster,
     _OrgCluster,
     _OrgUserRoleType,
+    _Project,
+    _ProjectUser,
+    _ProjectUserRoleType,
     _Quota,
+    _ResourcePreset,
+    _TPUPreset,
+    _VCDCloudProviderOptions,
 )
 
 from neuro_cli.formatters.config import BalanceFormatter
@@ -30,12 +35,16 @@ from neuro_cli.formatters.config import BalanceFormatter
 from .click_types import MEMORY
 from .defaults import JOB_CPU_NUMBER, JOB_MEMORY_AMOUNT, PRESET_PRICE
 from .formatters.admin import (
+    CloudProviderOptionsFormatter,
     ClustersFormatter,
     ClusterUserFormatter,
     OrgClusterFormatter,
     OrgClustersFormatter,
     OrgsFormatter,
     OrgUserFormatter,
+    ProjectFormatter,
+    ProjectsFormatter,
+    ProjectUserFormatter,
 )
 from .formatters.config import AdminQuotaFormatter
 from .root import Root
@@ -58,10 +67,10 @@ async def get_clusters(root: Root) -> None:
     """
     fmt = ClustersFormatter()
     with root.status("Fetching the list of clusters"):
-        config_clusters = await root.client._admin.list_config_clusters()
+        config_clusters = await root.client._clusters.list()
         admin_clusters = await root.client._admin.list_clusters()
     clusters: Dict[str, Tuple[Optional[_Cluster], Optional[_ConfigCluster]]] = {}
-    for config_cluster in config_clusters.values():
+    for config_cluster in config_clusters:
         clusters[config_cluster.name] = (None, config_cluster)
     for admin_cluster in admin_clusters:
         if admin_cluster.name in clusters:
@@ -75,6 +84,32 @@ async def get_clusters(root: Root) -> None:
         root.print(fmt(clusters))
 
 
+@command()
+@option(
+    "--idle-size",
+    type=int,
+    metavar="NUMBER",
+    help="Number of idle nodes in the node pool.",
+)
+@argument("cluster_name", required=True, type=str)
+@argument("node_pool_name", required=True, type=str)
+async def update_node_pool(
+    root: Root, cluster_name: str, node_pool_name: str, idle_size: Optional[int]
+) -> None:
+    """
+    Update cluster node pool.
+    """
+    await root.client._clusters.update_node_pool(
+        cluster_name, node_pool_name, idle_size=idle_size
+    )
+    if not root.quiet:
+        root.print(
+            f"Cluster [bold]{cluster_name}[/bold] "
+            f"node pool [bold]{node_pool_name}[/bold] successfully updated",
+            markup=True,
+        )
+
+
 @command(hidden=True)
 async def get_admin_clusters(root: Root) -> None:
     """
@@ -145,7 +180,7 @@ async def add_cluster(
     )
     if skip_provisioning:
         return
-    await root.client._admin.setup_cluster_cloud_provider(cluster_name, config_dict)
+    await root.client._clusters.setup_cluster_cloud_provider(cluster_name, config_dict)
     if not root.quiet:
         root.print(
             f"Cluster {cluster_name} successfully added "
@@ -229,12 +264,11 @@ async def show_cluster_options(root: Root, type: str) -> None:
     """
     Show available cluster options.
     """
-    config_options = await root.client._admin.get_cloud_provider_options(type)
-    root.print(
-        json.dumps(config_options, sort_keys=True, indent=2),
-        crop=False,
-        overflow="ignore",
+    fmt = CloudProviderOptionsFormatter()
+    options = await root.client._clusters.get_cloud_provider_options(
+        _CloudProviderType(type)
     )
+    root.print(fmt(options))
 
 
 @command()
@@ -459,8 +493,12 @@ storage:
 
 async def generate_vcd(root: Root, session: PromptSession[str]) -> str:
     args = {}
-    cloud_providers = await root.client._admin.list_cloud_providers()
-    cloud_providers = {k: v for k, v in cloud_providers.items() if k.startswith("vcd_")}
+    cloud_provider_options = await root.client._clusters.list_cloud_provider_options()
+    cloud_providers = {
+        c.type.value: c
+        for c in cloud_provider_options
+        if isinstance(c, _VCDCloudProviderOptions)
+    }
 
     if len(cloud_providers) == 1:
         args["vcd_provider"] = next(iter(cloud_providers.keys()))[4:]
@@ -472,13 +510,13 @@ async def generate_vcd(root: Root, session: PromptSession[str]) -> str:
 
     args["url"] = await session.prompt_async(
         "Url: ",
-        default=os.environ.get("VCD_URL", cloud_provider.get("url", "")),
+        default=os.environ.get(
+            "VCD_URL", str(cloud_provider.url) if cloud_provider.url else ""
+        ),
     )
     args["organization"] = await session.prompt_async(
         "Organization: ",
-        default=os.environ.get(
-            "VCD_ORGANIZATION", cloud_provider.get("organization", "")
-        ),
+        default=os.environ.get("VCD_ORGANIZATION", cloud_provider.organization or ""),
     )
     args["virtual_data_center"] = await session.prompt_async(
         "Virtual data center: ",
@@ -492,26 +530,26 @@ async def generate_vcd(root: Root, session: PromptSession[str]) -> str:
     )
     args["edge_name"] = await session.prompt_async(
         "Edge name: ",
-        default=cloud_provider.get("edge_name_template", "").format(
+        default=(cloud_provider.edge_name_template or "").format(
             organization=args["organization"], vdc=args["virtual_data_center"]
         ),
     )
     args["edge_ip"] = await session.prompt_async("Edge IP: ")
     args["edge_external_network_name"] = await session.prompt_async(
         "Edge external network: ",
-        default=cloud_provider.get("edge_external_network_name", ""),
+        default=cloud_provider.edge_external_network_name or "",
     )
     args["catalog_name"] = await session.prompt_async(
-        "Catalog: ", default=cloud_provider.get("catalog_name", "")
+        "Catalog: ", default=cloud_provider.catalog_name or ""
     )
     args["storage_profile_name"] = await session.prompt_async(
         "Storage profile: ",
-        default=(cloud_provider.get("storage_profile_names") or [""])[0],
+        default=(cloud_provider.storage_profile_names or [""])[0],
     )
     storage_size_gb = await session.prompt_async("Storage size (Gb): ")
     args["storage_size"] = storage_size_gb * 10**9
-    args["kubernetes_node_pool_id"] = cloud_provider["kubernetes_node_pool_id"]
-    args["platform_node_pool_id"] = cloud_provider["platform_node_pool_id"]
+    args["kubernetes_node_pool_id"] = cloud_provider.kubernetes_node_pool_id
+    args["platform_node_pool_id"] = cloud_provider.platform_node_pool_id
     return VCD_TEMPLATE.format_map(args)
 
 
@@ -874,38 +912,6 @@ async def add_user_credits(
     root.print(fmt(user_with_quota.balance))
 
 
-async def _update_presets_and_fetch(root: Root, presets: Mapping[str, Preset]) -> None:
-    cluster_name = root.client.config.cluster_name
-    await root.client._admin.update_cluster_resource_presets(cluster_name, presets)
-
-    if root.verbosity >= 1:
-        _print = root.print
-    else:
-
-        def _print(*args: Any, **kwargs: Any) -> None:
-            pass
-
-    _print("Requested presets update")
-
-    async def _sync_local_config() -> None:
-        _print("Fetching new server config", end="")
-        try:
-            while dict(root.client.config.presets) != presets:
-                _print(".", end="")
-                await root.client.config.fetch()
-                await asyncio.sleep(0.5)
-        finally:
-            _print("")
-
-    try:
-        await asyncio.wait_for(_sync_local_config(), 10)
-    except asyncio.TimeoutError:
-        log.warning(
-            "Fetched server presets are not same as new values. "
-            "Maybe there was some concurrent update?"
-        )
-
-
 @command()
 @argument("preset_name")
 @option(
@@ -986,18 +992,25 @@ async def add_resource_preset(
     presets = dict(root.client.config.presets)
     if preset_name in presets:
         raise ValueError(f"Preset '{preset_name}' already exists")
-    presets[preset_name] = Preset(
+    if tpu_type and tpu_software_version:
+        tpu_preset = _TPUPreset(type=tpu_type, software_version=tpu_software_version)
+    else:
+        tpu_preset = None
+    preset = _ResourcePreset(
+        name=preset_name,
         credits_per_hour=_parse_finite_decimal(credits_per_hour),
         cpu=cpu,
         memory=memory,
         gpu=gpu,
         gpu_model=gpu_model,
-        tpu_type=tpu_type,
-        tpu_software_version=tpu_software_version,
+        tpu=tpu_preset,
         scheduler_enabled=scheduler,
         preemptible_node=preemptible_node,
     )
-    await _update_presets_and_fetch(root, presets)
+    await root.client._clusters.add_resource_preset(
+        root.client.config.cluster_name, preset
+    )
+    await root.client.config.fetch()
     if not root.quiet:
         root.print(
             f"Added resource preset [b]{rich_escape(preset_name)}[/b] "
@@ -1052,10 +1065,12 @@ async def add_resource_preset(
     "--scheduler/--no-scheduler",
     "-p/-P",
     help="Use round robin scheduler for jobs",
+    default=None,
 )
 @option(
     "--preemptible-node/--non-preemptible-node",
     help="Use a lower-cost preemptible instance",
+    default=None,
 )
 async def update_resource_preset(
     root: Root,
@@ -1093,11 +1108,28 @@ async def update_resource_preset(
         "preemptible_node": preemptible_node,
     }
     kwargs = {key: value for key, value in kwargs.items() if value is not None}
-
-    presets[preset_name] = replace(preset, **kwargs)
-
-    await _update_presets_and_fetch(root, presets)
-
+    preset = replace(preset, **kwargs)
+    if preset.tpu_type and preset.tpu_software_version:
+        tpu_preset = _TPUPreset(
+            type=preset.tpu_type, software_version=preset.tpu_software_version
+        )
+    else:
+        tpu_preset = None
+    await root.client._clusters.update_resource_preset(
+        root.client.config.cluster_name,
+        _ResourcePreset(
+            name=preset_name,
+            credits_per_hour=preset.credits_per_hour,
+            cpu=preset.cpu,
+            memory=preset.memory,
+            gpu=preset.gpu,
+            gpu_model=preset.gpu_model,
+            tpu=tpu_preset,
+            scheduler_enabled=preset.scheduler_enabled,
+            preemptible_node=preset.preemptible_node,
+        ),
+    )
+    await root.client.config.fetch()
     if not root.quiet:
         root.print(
             f"Updated resource preset [b]{rich_escape(preset_name)}[/b] "
@@ -1116,7 +1148,10 @@ async def remove_resource_preset(root: Root, preset_name: str) -> None:
     if preset_name not in presets:
         raise ValueError(f"Preset '{preset_name}' not found")
     del presets[preset_name]
-    await _update_presets_and_fetch(root, presets)
+    await root.client._clusters.remove_resource_preset(
+        root.client.config.cluster_name, preset_name
+    )
+    await root.client.config.fetch()
     if not root.quiet:
         root.print(
             f"Removed resource preset [b]{rich_escape(preset_name)}[/b] "
@@ -1635,6 +1670,337 @@ async def add_org_cluster_credits(
     root.print(fmt(org.balance))
 
 
+# Projects
+
+
+@command()
+@argument("cluster_name", required=True, type=str)
+@option(
+    "--org",
+    metavar="ORG",
+    default=None,
+    type=str,
+    help="org name for org-cluster projects",
+)
+async def get_projects(
+    root: Root, cluster_name: str, org: Optional[str] = None
+) -> None:
+    """
+    Print the list of all projects in the cluster
+    """
+    fmt = ProjectsFormatter()
+    with root.status(f"Fetching the list of projects of cluster [b]{cluster_name}[/b]"):
+        org_clusters = await root.client._admin.list_projects(
+            cluster_name=cluster_name, org_name=org
+        )
+    with root.pager():
+        root.print(fmt(org_clusters))
+
+
+@command()
+@argument("cluster_name", required=True, type=str)
+@argument("name", required=True, type=str)
+@option(
+    "--org",
+    metavar="ORG",
+    default=None,
+    type=str,
+    help="org name for org-cluster projects",
+)
+@option(
+    "--default-role",
+    default=_ProjectUserRoleType.WRITER.value,
+    metavar="[ROLE]",
+    type=click.Choice([str(role) for role in list(_ProjectUserRoleType)]),
+    show_default=True,
+    help="Default role for new users added to project",
+)
+@option(
+    "--default",
+    is_flag=True,
+    help="Is this project is default, e.g. new cluster users will be automatically "
+    "added to it",
+)
+async def add_project(
+    root: Root,
+    name: str,
+    cluster_name: str,
+    org: Optional[str],
+    default_role: str,
+    default: bool = False,
+) -> None:
+    """
+    Add new project to specified cluster.
+
+    """
+
+    project = await root.client._admin.create_project(
+        name=name,
+        cluster_name=cluster_name,
+        org_name=org,
+        default_role=_ProjectUserRoleType(default_role),
+        is_default=default,
+    )
+    if not root.quiet:
+        root.print(
+            f"Added project [bold]{rich_escape(project.name)}[/bold] to cluster "
+            f"[bold]{rich_escape(cluster_name)}[/bold]"
+            f"{f' to org [bold]{rich_escape(org)}[/bold]' if org else ''}. Info:",
+            markup=True,
+        )
+        fmt = ProjectFormatter()
+        root.print(fmt(project, skip_cluster_org=True))
+
+
+@command()
+@argument("cluster_name", required=True, type=str)
+@argument("name", required=True, type=str)
+@option(
+    "--org",
+    metavar="ORG",
+    default=None,
+    type=str,
+    help="org name for org-cluster projects",
+)
+@option(
+    "--default-role",
+    default=_ProjectUserRoleType.WRITER.value,
+    metavar="[ROLE]",
+    type=click.Choice([str(role) for role in list(_ProjectUserRoleType)]),
+    show_default=True,
+    help="Default role for new users added to project",
+)
+@option(
+    "--default",
+    is_flag=True,
+    help="Is this project is default, e.g. new cluster users will be automatically "
+    "added to it",
+)
+async def update_project(
+    root: Root,
+    name: str,
+    cluster_name: str,
+    org: Optional[str],
+    default_role: str,
+    default: bool = False,
+) -> None:
+    """
+    Update project settings.
+
+    """
+    project = _Project(
+        name=name,
+        cluster_name=cluster_name,
+        org_name=org,
+        default_role=_ProjectUserRoleType(default_role),
+        is_default=default,
+    )
+    await root.client._admin.update_project(project)
+    if not root.quiet:
+        root.print(
+            f"Project [bold]{rich_escape(project.name)}[/bold] in cluster "
+            f"[bold]{rich_escape(cluster_name)}[/bold] "
+            f"{f'in org [bold]{rich_escape(org)}[/bold]' if org else ''} was "
+            f"updated. Info:",
+            markup=True,
+        )
+        fmt = ProjectFormatter()
+        root.print(fmt(project, skip_cluster_org=True))
+
+
+@command(hidden=True)
+@option("--force", default=False, help="Skip prompt", is_flag=True)
+@argument("cluster_name", required=True, type=str)
+@argument("name", required=True, type=str)
+@option(
+    "--org",
+    metavar="ORG",
+    default=None,
+    type=str,
+    help="org name for org-cluster projects",
+)
+async def remove_project(
+    root: Root, name: str, cluster_name: str, org: Optional[str], force: bool
+) -> None:
+    """
+    Drop a project
+
+    Completely removes project from the cluster.
+    """
+
+    if not force:
+        with patch_stdout():
+            answer: str = await PromptSession().prompt_async(
+                f"Are you sure that you want to drop project '{name}' "
+                f"from cluster '{cluster_name}' {f'in org {org}' if org else ''} (y/n)?"
+            )
+        if answer != "y":
+            return
+    await root.client._admin.delete_project(
+        project_name=name, cluster_name=cluster_name, org_name=org
+    )
+
+
+@command()
+@argument("cluster_name", required=True, type=str)
+@argument("project_name", required=True, type=str)
+@option(
+    "--org",
+    metavar="ORG",
+    default=None,
+    type=str,
+    help="org name for org-cluster projects",
+)
+async def get_project_users(
+    root: Root, cluster_name: str, project_name: str, org: Optional[str]
+) -> None:
+    """
+    List users in specified project
+    """
+    fmt = ProjectUserFormatter()
+    with root.status(
+        f"Fetching the list of project users of project [b]{project_name}[/b]"
+    ):
+        users = await root.client._admin.list_project_users(
+            project_name=project_name,
+            cluster_name=cluster_name,
+            org_name=org,
+            with_user_info=True,
+        )
+    with root.pager():
+        root.print(fmt(users))
+
+
+@command()
+@argument("cluster_name", required=True, type=str)
+@argument("project_name", required=True, type=str)
+@option(
+    "--org",
+    metavar="ORG",
+    default=None,
+    type=str,
+    help="org name for org-cluster projects",
+)
+@argument("user_name", required=True, type=str)
+@argument(
+    "role",
+    required=False,
+    default=None,
+    metavar="[ROLE]",
+    type=click.Choice([str(role) for role in list(_ProjectUserRoleType)]),
+)
+async def add_project_user(
+    root: Root,
+    cluster_name: str,
+    project_name: str,
+    org: Optional[str],
+    user_name: str,
+    role: Optional[None],
+) -> None:
+    """
+    Add user access to specified project.
+
+    The command supports one of 4 user roles: reader, writer, manager or admin.
+    """
+    user = await root.client._admin.create_project_user(
+        project_name=project_name,
+        cluster_name=cluster_name,
+        org_name=org,
+        user_name=user_name,
+        role=_ProjectUserRoleType(role) if role else None,
+    )
+    if not root.quiet:
+        root.print(
+            f"Added [bold]{rich_escape(user.user_name)}[/bold] to project "
+            f"[bold]{rich_escape(project_name)}[/bold] as "
+            f"[bold]{rich_escape(user.role)}[/bold]",
+            markup=True,
+        )
+
+
+@command()
+@argument("cluster_name", required=True, type=str)
+@argument("project_name", required=True, type=str)
+@option(
+    "--org",
+    metavar="ORG",
+    default=None,
+    type=str,
+    help="org name for org-cluster projects",
+)
+@argument("user_name", required=True, type=str)
+@argument(
+    "role",
+    required=True,
+    metavar="[ROLE]",
+    type=click.Choice([str(role) for role in list(_ProjectUserRoleType)]),
+)
+async def update_project_user(
+    root: Root,
+    cluster_name: str,
+    project_name: str,
+    org: Optional[str],
+    user_name: str,
+    role: str,
+) -> None:
+    """
+    Update user access to specified project.
+
+    The command supports one of 4 user roles: reader, writer, manager or admin.
+    """
+    user = _ProjectUser(
+        project_name=project_name,
+        cluster_name=cluster_name,
+        org_name=org,
+        user_name=user_name,
+        role=_ProjectUserRoleType(role),
+    )
+
+    await root.client._admin.update_project_user(user)
+    if not root.quiet:
+        root.print(
+            f"Update [bold]{rich_escape(user.user_name)}[/bold] role in project "
+            f"[bold]{rich_escape(project_name)}[/bold] as "
+            f"[bold]{rich_escape(user.role)}[/bold]",
+            markup=True,
+        )
+
+
+@command()
+@argument("cluster_name", required=True, type=str)
+@argument("project_name", required=True, type=str)
+@option(
+    "--org",
+    metavar="ORG",
+    default=None,
+    type=str,
+    help="org name for org-cluster projects",
+)
+@argument("user_name", required=True, type=str)
+async def remove_project_user(
+    root: Root,
+    cluster_name: str,
+    project_name: str,
+    org: Optional[str],
+    user_name: str,
+) -> None:
+    """
+    Remove user access from the project.
+    """
+    await root.client._admin.delete_project_user(
+        project_name=project_name,
+        cluster_name=cluster_name,
+        org_name=org,
+        user_name=user_name,
+    )
+    if not root.quiet:
+        root.print(
+            f"Removed [bold]{rich_escape(user_name)}[/bold] from project "
+            f"[bold]{rich_escape(project_name)}[/bold]",
+            markup=True,
+        )
+
+
 admin.add_command(get_clusters)
 admin.add_command(get_admin_clusters)
 admin.add_command(generate_cluster_config)
@@ -1642,6 +2008,7 @@ admin.add_command(add_cluster)
 admin.add_command(update_cluster)
 admin.add_command(remove_cluster)
 admin.add_command(show_cluster_options)
+admin.add_command(update_node_pool)
 
 admin.add_command(get_cluster_users)
 admin.add_command(add_cluster_user)
@@ -1674,3 +2041,13 @@ admin.add_command(get_org_cluster_quota)
 admin.add_command(set_org_cluster_quota)
 admin.add_command(set_org_cluster_credits)
 admin.add_command(add_org_cluster_credits)
+
+admin.add_command(get_projects)
+admin.add_command(add_project)
+admin.add_command(update_project)
+admin.add_command(remove_project)
+
+admin.add_command(get_project_users)
+admin.add_command(add_project_user)
+admin.add_command(update_project_user)
+admin.add_command(remove_project_user)
