@@ -21,6 +21,7 @@ from prompt_toolkit.key_binding.key_processor import KeyPressEvent
 from prompt_toolkit.keys import Keys
 from prompt_toolkit.output import Output, create_output
 from prompt_toolkit.shortcuts import PromptSession
+from rich.markup import escape as rich_escape
 
 from neuro_sdk import (
     JobDescription,
@@ -37,7 +38,10 @@ from .root import Root
 log = logging.getLogger(__name__)
 
 
-JOB_STARTED = "[dim]===== Job is running, press Ctrl-C to detach/kill =====[/dim]"
+JOB_STARTED_NEURO_HAS_TTY = (
+    "[green]√[/green] "
+    "[dim]===== Job is running, press Ctrl-C to detach/kill =====[/dim]"
+)
 
 JOB_STARTED_NEURO_HAS_NO_TTY = (
     "[dim]===== Job is running, press Ctrl-C to detach =====[/dim]"
@@ -66,6 +70,7 @@ class InterruptAction(enum.Enum):
 class AttachHelper:
     attach_ready: bool
     log_printed: bool
+    job_started_msg: str
     write_sem: asyncio.Semaphore
     quiet: bool
     action: InterruptAction
@@ -73,6 +78,7 @@ class AttachHelper:
     def __init__(self, *, quiet: bool) -> None:
         self.attach_ready = False
         self.log_printed = False
+        self.job_started_msg = ""
         self.write_sem = asyncio.Semaphore()
         self.quiet = quiet
         self.action = InterruptAction.NOTHING
@@ -109,7 +115,10 @@ async def process_logs(
                 if helper.attach_ready:
                     return
                 async with helper.write_sem:
-                    helper.log_printed = True
+                    if not helper.log_printed:
+                        if not root.quiet:
+                            root.print(helper.job_started_msg, markup=True)
+                        helper.log_printed = True
                     sys.stdout.write(txt)
                     sys.stdout.flush()
             else:
@@ -128,6 +137,10 @@ async def process_exec(
     finally:
         root.soft_reset_tty()
 
+    if not root.quiet:
+        status = await root.client.jobs.status(job)
+        print_job_result(root, status)
+
     sys.exit(exit_code)
 
 
@@ -243,8 +256,6 @@ async def process_attach(
                     pass
                 else:
                     break
-        else:
-            break
 
 
 async def _process_attach_single_try(
@@ -257,113 +268,118 @@ async def _process_attach_single_try(
     # Note, the job should be in running/finished state for this call,
     # passing pending job is forbidden
 
-    async with AsyncExitStack() as stack:
-        for local_port, job_port in port_forward:
-            root.print(
-                f"Port localhost:{local_port} will be forwarded to port {job_port}"
-            )
-            await stack.enter_async_context(
-                root.client.jobs.port_forward(
-                    job.id, local_port, job_port, cluster_name=job.cluster_name
+    while True:
+        restarts = job.history.restarts
+        async with AsyncExitStack() as stack:
+            for local_port, job_port in port_forward:
+                root.print(
+                    f"Port localhost:{local_port} will be forwarded to port {job_port}"
+                )
+                await stack.enter_async_context(
+                    root.client.jobs.port_forward(
+                        job.id, local_port, job_port, cluster_name=job.cluster_name
+                    )
                 )
-            )
 
-        try:
+            helper = AttachHelper(quiet=root.quiet)
             if tty:
-                action = await _attach_tty(
-                    root, job.id, logs, cluster_name=job.cluster_name
+                helper.job_started_msg = JOB_STARTED_TTY
+            elif root.tty:
+                helper.job_started_msg = JOB_STARTED_NEURO_HAS_TTY
+            else:
+                helper.job_started_msg = JOB_STARTED_NEURO_HAS_NO_TTY
+            loop = asyncio.get_event_loop()
+            if logs:
+                logs_printer = loop.create_task(
+                    process_logs(root, job.id, helper, cluster_name=job.cluster_name)
                 )
             else:
-                action = await _attach_non_tty(
-                    root, job.id, logs, cluster_name=job.cluster_name
-                )
+                # Placeholder, prints nothing
+                logs_printer = loop.create_task(asyncio.sleep(0))
+
+            try:
+                job = await root.client.jobs.status(job.id)
+                if job.status.is_finished:
+                    await logs_printer
+                    print_job_result(root, job)
+                    if job.status == JobStatus.FAILED:
+                        sys.exit(job.history.exit_code or EX_PLATFORMERROR)
+                    else:
+                        sys.exit(job.history.exit_code)
+
+                if tty:
+                    action = await _attach_tty(
+                        root, job.id, helper, cluster_name=job.cluster_name
+                    )
+                else:
+                    action = await _attach_non_tty(
+                        root, job.id, helper, cluster_name=job.cluster_name
+                    )
 
-            with JobStopProgress.create(
-                console=root.console,
-                quiet=root.quiet,
-            ) as progress:
                 if action == InterruptAction.KILL:
-                    progress.kill(job)
+                    with JobStopProgress.create(
+                        root.console, quiet=root.quiet
+                    ) as progress:
+                        progress.kill(job)
                     sys.exit(128 + signal.SIGINT)
                 elif action == InterruptAction.DETACH:
-                    progress.detach(job)
+                    with JobStopProgress.create(
+                        root.console, quiet=root.quiet
+                    ) as progress:
+                        progress.detach(job)
                     sys.exit(0)
-        except ResourceNotFound:
-            # Container already stopped, so we can ignore such error.
-            pass
-        finally:
-            root.soft_reset_tty()
+            except ResourceNotFound:
+                # Container already stopped, so we can ignore such error.
+                pass
+            finally:
+                await root.cancel_with_logging(logs_printer)
+                root.soft_reset_tty()
 
-        # We exited attach function not because of detach or kill,
-        # probably we lost connectivity?
-        try:
-            job = await root.client.jobs.status(job.id)
-        except aiohttp.ClientConnectionError:
-            raise RetryAttach
-        # Maybe it is spurious disconnect, and we should re-attach back?
-        # Check container liveness by calling attach once
-        try:
-            async with root.client.jobs.attach(
-                job.id, stdin=True, cluster_name=job.cluster_name
-            ):
-                raise RetryAttach
-        except (asyncio.CancelledError, RetryAttach):
-            raise
-        except Exception:
-            pass  # Was unable to reconnect, most likely container is dead
-
-        # The class pins the current time in counstructor,
-        # that's why we need to initialize
-        # it AFTER the disconnection from attached session.
-        with JobStopProgress.create(
-            console=root.console,
-            quiet=root.quiet,
-        ) as progress:
-            while job.status == JobStatus.RUNNING:
-                await asyncio.sleep(0.2)
+            # We exited attach function not because of detach or kill,
+            # probably we lost connectivity?
+            try:
                 job = await root.client.jobs.status(job.id)
-                if not progress.step(job):
-                    sys.exit(EX_IOERR)
-            if job.status == JobStatus.FAILED:
-                sys.exit(job.history.exit_code or EX_PLATFORMERROR)
-            sys.exit(job.history.exit_code)
+            except aiohttp.ClientConnectionError:
+                raise RetryAttach
+            # Maybe it is spurious disconnect, and we should re-attach back?
+            # Check container liveness by calling attach once
+            try:
+                async with root.client.jobs.attach(
+                    job.id, stdin=True, cluster_name=job.cluster_name
+                ):
+                    raise RetryAttach
+            except (asyncio.CancelledError, RetryAttach):
+                raise
+            except Exception:
+                pass  # Was unable to reconnect, most likely container is dead
+
+            # The class pins the current time in counstructor,
+            # that's why we need to initialize
+            # it AFTER the disconnection from attached session.
+            with JobStopProgress.create(root.console, quiet=root.quiet) as progress:
+                while (not job.status.is_finished) and (
+                    job.history.reason == "Restarting"
+                    or job.history.restarts == restarts
+                ):
+                    if not progress.step(job):
+                        sys.exit(EX_IOERR)
+                    await asyncio.sleep(0.2)
+                    job = await root.client.jobs.status(job.id)
+                progress.end(job)
 
 
 async def _attach_tty(
-    root: Root, job: str, logs: bool, *, cluster_name: Optional[str]
+    root: Root, job: str, helper: AttachHelper, *, cluster_name: Optional[str]
 ) -> InterruptAction:
-    if not root.quiet:
-        root.print(JOB_STARTED_TTY, markup=True)
-
-    loop = asyncio.get_event_loop()
-    helper = AttachHelper(quiet=root.quiet)
-
     stdout = create_output()
     h, w = stdout.get_size()
 
-    if logs:
-        logs_printer = loop.create_task(
-            process_logs(root, job, helper, cluster_name=cluster_name)
-        )
-    else:
-        # Placeholder, prints nothing
-        logs_printer = loop.create_task(asyncio.sleep(0))
-
     async with root.client.jobs.attach(
         job, tty=True, stdin=True, stdout=True, stderr=False, cluster_name=cluster_name
     ) as stream:
-        status = await root.client.jobs.status(job)
-
-        if status.status is not JobStatus.RUNNING:
-            # Job is finished
-            await logs_printer
-            if status.status == JobStatus.FAILED:
-                sys.exit(status.history.exit_code or EX_PLATFORMERROR)
-            else:
-                sys.exit(status.history.exit_code)
-
         await stream.resize(h=h, w=w)
 
+        loop = asyncio.get_event_loop()
         resize_task = loop.create_task(_process_resizing(stream.resize, stdout))
         input_task = loop.create_task(_process_stdin_tty(stream, helper))
         output_task = loop.create_task(
@@ -377,7 +393,6 @@ async def _attach_tty(
             await root.cancel_with_logging(resize_task)
             await root.cancel_with_logging(input_task)
             await _cancel_attach_output(root, output_task)
-            await root.cancel_with_logging(logs_printer)
 
         return helper.action
 
@@ -484,52 +499,21 @@ async def _process_stdout_tty(
         else:
             txt = decoder.decode(chunk.data)
         async with helper.write_sem:
-            if not helper.quiet and not helper.attach_ready:
-                # Print header to stdout only,
-                # logs are printed to stdout and never to
-                # stderr (but logs printing is stopped by
-                # helper.attach_ready = True regardless
-                # what stream had receive text in attached mode.
-                if helper.log_printed:
-                    s = ATTACH_STARTED_AFTER_LOGS
-                    if root.tty:
-                        s = "[green]√[/green] " + s
-                    root.print(s, markup=True)
-            helper.attach_ready = True
+            if not helper.attach_ready:
+                await _print_header(root, helper)
+                helper.attach_ready = True
             stdout.write_raw(txt)
             stdout.flush()
 
 
 async def _attach_non_tty(
-    root: Root, job: str, logs: bool, *, cluster_name: Optional[str]
+    root: Root, job: str, helper: AttachHelper, *, cluster_name: Optional[str]
 ) -> InterruptAction:
-    if not root.quiet:
-        s = JOB_STARTED_NEURO_HAS_NO_TTY
-        if root.tty:
-            s = "[green]√[/green] " + JOB_STARTED
-        root.print(s, markup=True)
-
-    loop = asyncio.get_event_loop()
-    helper = AttachHelper(quiet=root.quiet)
-
-    if logs:
-        logs_printer = loop.create_task(
-            process_logs(root, job, helper, cluster_name=cluster_name)
-        )
-    else:
-        # Placeholder, prints nothing
-        logs_printer = loop.create_task(asyncio.sleep(0))
-
     async with root.client.jobs.attach(
         job, stdin=True, stdout=True, stderr=True, cluster_name=cluster_name
     ) as stream:
-        status = await root.client.jobs.status(job)
-        if status.history.exit_code is not None:
-            # Wait for logs printing finish before exit
-            await logs_printer
-            sys.exit(status.history.exit_code)
-
         input_task = None
+        loop = asyncio.get_event_loop()
         if root.tty:
             input_task = loop.create_task(_process_stdin_non_tty(root, stream))
         output_task = loop.create_task(_process_stdout_non_tty(root, stream, helper))
@@ -545,7 +529,6 @@ async def _attach_non_tty(
                 await root.cancel_with_logging(input_task)
             await _cancel_attach_output(root, output_task)
             await root.cancel_with_logging(ctrl_c_task)
-            await root.cancel_with_logging(logs_printer)
         return helper.action
 
 
@@ -580,18 +563,9 @@ async def _process_stdout_non_tty(
     async def _write(fileno: int, txt: str) -> None:
         f = streams[fileno]
         async with helper.write_sem:
-            if not helper.quiet and not helper.attach_ready:
-                # Print header to stdout only,
-                # logs are printed to stdout and never to
-                # stderr (but logs printing is stopped by
-                # helper.attach_ready = True regardless
-                # what stream had receive text in attached mode.
-                if helper.log_printed:
-                    s = ATTACH_STARTED_AFTER_LOGS
-                    if root.tty:
-                        s = "[green]√[/green] " + s
-                    root.print(s, markup=True)
-            helper.attach_ready = True
+            if not helper.attach_ready:
+                await _print_header(root, helper)
+                helper.attach_ready = True
             f.write(txt)
             f.flush()
 
@@ -608,6 +582,23 @@ async def _process_stdout_non_tty(
             await _write(chunk.fileno, txt)
 
 
+async def _print_header(root: Root, helper: AttachHelper) -> None:
+    if not helper.quiet and not helper.attach_ready:
+        # Print header to stdout only,
+        # logs are printed to stdout and never to
+        # stderr (but logs printing is stopped by
+        # helper.attach_ready = True regardless
+        # what stream had receive text in attached mode.
+        if helper.log_printed:
+            s = ATTACH_STARTED_AFTER_LOGS
+            if root.tty:
+                s = "[green]√[/green] " + s
+            root.print(s, markup=True)
+        else:
+            if not root.quiet:
+                root.print(helper.job_started_msg, markup=True)
+
+
 def _create_interruption_dialog() -> PromptSession[InterruptAction]:
     bindings = KeyBindings()
 
@@ -701,3 +692,25 @@ async def _cancel_attach_output(root: Root, output_task: "asyncio.Task[Any]") ->
         if ex and isinstance(ex, StdStreamError):
             return
     await root.cancel_with_logging(output_task)
+
+
+def print_job_result(root: Root, job: JobDescription) -> None:
+    if job.status == JobStatus.SUCCEEDED and root.verbosity > 0:
+        msg = f"Job [b]{job.id}[/b] finished successfully"
+        if root.tty:
+            msg = "[green]√[/green] " + msg
+        root.print(msg, markup=True)
+    if job.status == JobStatus.CANCELLED and root.verbosity >= 0:
+        msg = f"Job [b]{job.id}[/b] was cancelled"
+        if root.tty:
+            msg = "[green]√[/green] " + msg
+        if job.history.reason:
+            msg += f" ({rich_escape(job.history.reason)})"
+        root.print(msg, markup=True)
+    if job.status == JobStatus.FAILED and root.verbosity >= 0:
+        msg = f"Job [b]{job.id}[/b] failed"
+        if root.tty:
+            msg = "[red]×[/red] " + msg
+        if job.history.reason:
+            msg += f" ({rich_escape(job.history.reason)})"
+        root.print(msg, markup=True)
