@@ -1,6 +1,8 @@
+import calendar
 import operator
+from datetime import time
 from decimal import Decimal
-from typing import Iterable, List, Mapping, Optional, Union
+from typing import Iterable, List, Mapping, Optional, Sequence, Union
 
 import click
 from rich import box
@@ -10,7 +12,7 @@ from rich.padding import Padding
 from rich.table import Table
 from rich.text import Text
 
-from neuro_sdk import Cluster, Config, Preset, Quota, _Balance, _Quota
+from neuro_sdk import Cluster, Config, Preset, Quota, _Balance, _ConfigCluster, _Quota
 
 from neuro_cli.click_types import OrgType
 from neuro_cli.utils import format_size
@@ -23,6 +25,7 @@ class ConfigFormatter:
         available_jobs_counts: Mapping[str, int],
         quota: Quota,
         org_quota: Optional[Quota],
+        config_cluster: Optional[_ConfigCluster] = None,
     ) -> RenderableType:
         table = Table(
             title="User Configuration:",
@@ -46,10 +49,10 @@ class ConfigFormatter:
         table.add_row("API URL", str(config.api_url))
         table.add_row("Docker Registry URL", str(config.registry_url))
 
-        return RichGroup(
-            table,
-            _format_presets(config.presets, available_jobs_counts),
-        )
+        to_render = [table, _format_presets(config.presets, available_jobs_counts)]
+        if config_cluster and config_cluster.energy:
+            to_render.extend(_format_cluster_energy(config_cluster))  # type: ignore
+        return RichGroup(*to_render)
 
 
 class AdminQuotaFormatter:
@@ -158,6 +161,51 @@ def _format_presets(
     return table
 
 
+def _format_cluster_energy(cluster: _ConfigCluster) -> Sequence[RenderableType]:
+    assert cluster.energy
+
+    summary_tbl = Table(
+        title="Cluster energy parameters:",
+        title_justify="left",
+        box=None,
+        show_header=False,
+        show_edge=False,
+    )
+    summary_tbl.add_column()
+    summary_tbl.add_column(style="bold")
+    summary_tbl.add_row("CO2 eq g/kWh: ", str(cluster.energy.co2_grams_eq_per_kwh))
+    summary_tbl.add_row("Cluster location timezone: ", str(cluster.timezone))
+
+    schedules_tbl = Table(
+        title="Energy schedules:",
+        title_justify="left",
+        box=box.SIMPLE_HEAVY,
+        show_edge=False,
+    )
+    schedules_tbl.add_column("Name", style="bold", justify="left")
+    schedules_tbl.add_column("Price (kW/h)", justify="center")
+    schedules_tbl.add_column("Start time", justify="center")
+    schedules_tbl.add_column("End time", justify="center")
+    schedules_tbl.add_column("Weekday", justify="center")
+    for schedule in cluster.energy.schedules:
+        joint_periods: dict[tuple[time, time], list[int]] = {}
+        for period in schedule.periods:
+            key = (period.start_time, period.end_time)
+            if key not in joint_periods:
+                joint_periods[key] = []
+            joint_periods[key].append(period.weekday)
+
+        for timeslot, days in sorted(joint_periods.items(), key=lambda x: x[1]):
+            schedules_tbl.add_row(
+                str(schedule.name),
+                str(schedule.price_per_kwh),
+                timeslot[0].strftime("%H:%M"),
+                timeslot[1].strftime("%H:%M"),
+                ", ".join([calendar.day_abbr[x - 1] for x in sorted(days)]),
+            )
+    return summary_tbl, schedules_tbl
+
+
 class AliasesFormatter:
     def __call__(self, aliases: Iterable[click.Command]) -> Table:
         table = Table(box=box.MINIMAL_HEAVY_HEAD)
