@@ -38,7 +38,7 @@ from neuro_cli.formatters.utils import (
 from neuro_cli.parse_utils import parse_sort_keys
 from neuro_cli.utils import parse_org_name, resolve_disk
 
-from .ael import process_attach, process_exec, process_logs
+from .ael import print_job_result, process_attach, process_exec, process_logs
 from .click_types import (
     CLUSTER,
     JOB,
@@ -234,6 +234,9 @@ async def logs(root: Root, since: str, job: str, timestamps: bool) -> None:
         since=_parse_date(since),
         timestamps=timestamps,
     )
+    if not root.quiet:
+        status = await root.client.jobs.status(id)
+        print_job_result(root, status)
 
 
 @command()
@@ -258,11 +261,14 @@ async def attach(root: Root, job: str, port_forward: List[Tuple[int, int]]) -> N
         status=JobStatus.items(),
     )
     status = await root.client.jobs.status(id)
-    progress = JobStartProgress.create(console=root.console, quiet=root.quiet)
-    while status.status.is_pending:
-        await asyncio.sleep(0.2)
-        status = await root.client.jobs.status(id)
-        progress.step(status)
+    if status.status.is_pending:
+        with JobStartProgress.create(root.console, quiet=root.quiet) as progress:
+            progress.step(status)
+            while status.status.is_pending:
+                await asyncio.sleep(0.2)
+                status = await root.client.jobs.status(id)
+                progress.step(status)
+
     tty = status.container.tty
     _check_tty(root, tty)
 
@@ -1016,6 +1022,14 @@ async def kill(root: Root, jobs: Sequence[str]) -> None:
     "Jobs with higher priority will start before ones with lower priority. "
     "Priority should be supported by cluster.",
 )
+@option(
+    "--energy-schedule",
+    metavar="NAME",
+    help=(
+        "Run job only within a selected energy schedule. "
+        "Selected preset should have scheduler enabled."
+    ),
+)
 @TTY_OPT
 async def run(
     root: Root,
@@ -1049,6 +1063,7 @@ async def run(
     privileged: bool,
     share: Sequence[str],
     priority: Optional[str],
+    energy_schedule: Optional[str],
 ) -> None:
     """
     Run a job
@@ -1059,12 +1074,12 @@ async def run(
 
     Examples:
 
-    # Starts a container pytorch:latest on a machine with smaller GPU resources
+    # Starts a container pytorch/pytorch:latest on a machine with smaller GPU resources
     # (see exact values in `neuro config show`) and with two volumes mounted:
     #   storage:/<home-directory>   --> /var/storage/home (in read-write mode),
     #   storage:/neuromation/public --> /var/storage/neuromation (in read-only mode).
     neuro run --preset=gpu-small --volume=storage::/var/storage/home:rw \\\\
-        --volume=storage:/neuromation/public:/var/storage/home:ro pytorch:latest
+        --volume=storage:/neuromation/public:/var/storage/home:ro pytorch/pytorch:latest
 
     # Starts a container using the custom image my-ubuntu:latest stored in neuro
     # registry, run /script.sh and pass arg1 and arg2 as its arguments:
@@ -1117,6 +1132,7 @@ async def run(
         cluster_name=cluster_name,
         org_name=org_name,
         priority=priority,
+        energy_schedule_name=energy_schedule,
     )
 
 
@@ -1188,6 +1204,7 @@ async def run_job(
     cluster_name: str,
     org_name: Optional[str],
     priority: Optional[str],
+    energy_schedule_name: Optional[str],
 ) -> JobDescription:
     if http_auth is None:
         http_auth = True
@@ -1285,13 +1302,14 @@ async def run_job(
         schedule_timeout=job_schedule_timeout,
         privileged=privileged,
         priority=job_priority,
+        energy_schedule_name=energy_schedule_name,
     )
     permission = Permission(job.uri, Action.WRITE)
     for user in share:
         await root.client.users.share(user, permission)
     with JobStartProgress.create(console=root.console, quiet=root.quiet) as progress:
         progress.begin(job)
-        while wait_start and job.status == JobStatus.PENDING:
+        while wait_start and job.status.is_pending:
             await asyncio.sleep(0.2)
             job = await root.client.jobs.status(job.id)
             progress.step(job)
@@ -1461,6 +1479,8 @@ def _job_to_cli_args(job: JobDescription) -> List[str]:
         res += ["--privileged"]
     if job.priority != JobPriority.NORMAL:
         res += ["--priority", job.priority.name.lower()]
+    if job.energy_schedule_name:
+        res += ["--energy-schedule", job.energy_schedule_name]
     res += [str(job.container.image)]
     if job.container.command:
         res += [job.container.command]
